/*
 * Copyright 2004-2005 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

package com.sun.enterprise.ee.admin.mbeans;

import com.sun.enterprise.admin.mbeans.custom.BasicCustomMBeanConfigQueries;
import com.sun.enterprise.admin.mbeans.custom.CustomMBeanConfigQueries;
import com.sun.enterprise.admin.server.core.CustomMBeanException;
import com.sun.enterprise.admin.target.Target;
import com.sun.enterprise.admin.target.TargetBuilder;
import com.sun.enterprise.admin.target.TargetType;
import com.sun.enterprise.config.ConfigException;
import com.sun.enterprise.config.serverbeans.Mbean;
import com.sun.enterprise.config.serverbeans.Server;
import com.sun.enterprise.config.serverbeans.ServerBeansFactory;
import com.sun.enterprise.config.serverbeans.ServerHelper;
import java.util.List;
import javax.management.ObjectName;

public class EnterpriseCustomMBeanConfigQueries extends BasicCustomMBeanConfigQueries implements CustomMBeanConfigQueries {

    private final TargetType[] vTargets;
    /** Creates a new instance of EnterpriseCustomMBeanConfigQueries */
    public EnterpriseCustomMBeanConfigQueries() {
        super();
        vTargets    = new TargetType[4]; //domain, DAS, standalone server, cluster
        vTargets[0] = TargetType.DOMAIN;
        vTargets[1] = TargetType.DAS;
        vTargets[2]  = TargetType.SERVER;
        vTargets[3]  = TargetType.CLUSTER;
    }

    public boolean existsMBean(String target, String name) throws CustomMBeanException {
        return ( false) ; //TODO
    }

    public boolean isMBeanEnabled(String target, String name) throws CustomMBeanException {
        return ( false); //TODO
    }

    public List<ObjectName> listMBeanConfigObjectNames(String target) throws CustomMBeanException {
        /* note that all the servers in a cluster should have exactly same references to mbeans, hence getting a single server
         in a cluster is good enough.
         * <servers>
         *  <server name="s1">
         *   <application-ref ref="fooMBean" .../>
         *  </server>
         *  <server name="s2">
         *   <application-ref ref="fooMBean" .../>
         *  </server>
         * </servers>
         * <clusters>
         *  <cluster name="s1s2" ...>
         *   <server-ref>s1</server-ref>
         *   <server-ref>s2</server-ref>
         *  </cluster>
         * </clusters>
         */
        List<ObjectName> ons = null;
        Target t = null;
        try {
            t = TargetBuilder.INSTANCE.createTarget(target, vTargets, target, this.acc);
        } catch(ConfigException e) {
            throw new CustomMBeanException(e);
        }
        if (TargetType.DAS.equals(t.getType()) || TargetType.SERVER.equals(t.getType())) {
            if (TargetType.SERVER.equals(t.getType())) {
                EnterpriseCustomMBeanOperations.checkClusteredInstance(this.acc, t.getName());
            }
            ons = super.listMBeanConfigObjectNamesForServer(t.getName());
        }
        else if (TargetType.DOMAIN.equals(t.getType())) {
            // just list the mbean definitions
            ons = this.getMBeanDefinitionObjectNamesInDomain();
        }
        else {//has to be a cluster
            String nameOfAServerInCluster = null;
            try {
                Server[] servers= ServerHelper.getServersInCluster(this.acc, target);
                nameOfAServerInCluster = servers[0].getName();
                ons = super.listMBeanConfigObjectNamesForServer(nameOfAServerInCluster);
            } catch(ConfigException e) {
                throw new CustomMBeanException(e);
            }
        }
        return ( ons ) ;
    }

    public List<ObjectName> listMBeanConfigObjectNames(String target, int type, boolean state) throws CustomMBeanException {
        return ( null ); //TODO
    }

    public List<String> listMBeanNames(String target) throws CustomMBeanException {
        return ( super.listMBeanNames(target) );
    }
    
    ///// Private Methods
    private List<ObjectName> getMBeanDefinitionObjectNamesInDomain() throws CustomMBeanException {
        try {
            final List<Mbean> mbeans = ServerBeansFactory.getAllMBeanDefinitions(this.acc);
            return ( super.mbeans2ConfigMBeanObjectNames(mbeans) );
        } catch (final Exception e) {
            throw new CustomMBeanException(e);
        }
    }
}